#######################################################################################
"""
"""
from __future__ import nested_scopes#{{{
import sys

import os
from os import getcwd, environ,sep
from os.path import exists, normpath, dirname
import re

from java.lang import System, String, Runtime
from jarray import array

from java.util import Properties
from java.io import *
#}}}

def getProps(root):#{{{
    basedir = os.path.join(root, "bin", "packages")
    file = os.path.join(root,"bin", "packages", "properties.txt")
    if not exists(file):
	return None
    
    props = Properties()
    input = FileInputStream(File(file))
    props.load(input)
    input.close()
    return props
#}}}

def get_dependencies(root): # {{{
    props = getProps(root)
    if not props:
	return []
    pkgs = props["packages"].split(" ")
    
    return [ pkg for pkg in pkgs ]

def getPropArguments(props):
    s = []
    for key in props.keySet():
	s.append("-D"+key + "=" + props[key])
    return s
    
# }}}

def get_arch(): # {{{ this is actually os not arch!
    osname = System.getProperty("os.name")
    osname = osname.lower()
    if osname.startswith("windows"):
        return "win"
    elif osname.startswith("linux"):
	return "linux"
    elif osname.startswith("mac"):
	return "mac"
    elif osname.startswith("solaris"):
	return "sun"
    raise 'unknown arch %s' % (osname)

# }}}

def find_jars(pkgroot, pkgs):#{{{
    jars = []
    for pkg in pkgs:
	libdir = pkgroot + sep + pkg + sep + "lib"
	if not os.path.exists(libdir):
	    continue
	jars = jars + [ libdir + sep + f 
			for f in os.listdir(libdir) 
			if f.endswith('.jar') or f.endswith('.zip')]
	
    return jars	
#}}}

def get_class_path(pkgroot, dependencies):#{{{
    jars = find_jars(pkgroot, dependencies)
    classpath = os.pathsep.join(jars)
    return classpath
#}}}

def get_native_libpath(pkgroot, dependencies):#{{{
    dir = "lib" + System.getProperty("sun.arch.data.model") # 32 or 64
    # When building pakcages for mac, we copy universal binaries into lib32 directory instead of lib32 and 
    # lib64 dirs separately. Hence on mac machines, lib path should always point to lib32 directories.
    
    arch = get_arch()
    if arch == "mac":
	dir = "lib32"
    libpath=[]
    for pkg in dependencies:
	libpath +=  [pkgroot + sep + pkg + sep + dir]

    return os.pathsep.join(libpath)
#}}}

def launch():#{{{

    def outputLogging(istream, product_home,name):#{{{
	from java.io import BufferedReader, InputStreamReader
	r = BufferedReader (InputStreamReader(istream))
	out = open(os.path.join(product_home,name),"w")
	out.write("x"*50 + "\n")
	while 1:
	    l = r.readLine()
	    if not l:
		break
	    out.write(l + "\n")
	out.close()
    #}}} 
	    
    def run(cmd, **env):#{{{
	_env = os.environ
	_env.update(env)
	envarray = [ a + "=" + b for a,b in _env.items()]
	process = Runtime.getRuntime().exec(cmd, envarray)
	istream = process.getInputStream()
	errstream = process.getErrorStream()
	import thread
	outfile = File(sys.argv[1] + ".out.log")
	errfile = File(sys.argv[1] + ".err.log")
	thread.start_new_thread(outputLogging, (istream,product_home, outfile.getName()))

	thread.start_new_thread(outputLogging, (errstream,product_home, errfile.getName()))
    #}}}


    product_home = File(System.getProperty("product.home")).getCanonicalPath()
    pkgroot = os.path.join(product_home, "bin", "packages")

    dependencies = get_dependencies(product_home)

    libpath = get_native_libpath(pkgroot, dependencies)

    params = ["-Dproduct.home="+product_home]
    params += ["-Djava.library.path=" + libpath]
    params += ["-Dbindir=" + os.path.join(product_home,"bin","packages")]
    params += ["-Dbin.dir=" + os.path.join(product_home,"bin","packages")]
    props = getProps(product_home)
    for s in props['java.options'].split(" "):
	params += [s]
    params += getPropArguments(props)
    classpath = get_class_path(pkgroot, dependencies)
    jvm = os.path.join(System.getProperty("java.home"), "bin", "java")
    class_name = "com.strandgenomics.cube.framework.PackageLoader"
    cmdlist = [jvm, "-classpath", classpath ] + params 
    cmdlist += [class_name]
    for arg in sys.argv[1:]:
	cmdlist += [arg]
    run(cmdlist, path=libpath, LD_LIBRARY_PATH=libpath)

#}}}

launch()
